@@ -5,29 +5,25 @@
 #![feature(const_ptr_offset_from)]
 #![feature(const_raw_ptr_deref)]
 
-use components_arena::{Arena, Component, NewtypeComponentId, Id, ComponentClassMutex};
+use components_arena::{Arena, Component, NewtypeComponentId, Id};
 use dep_obj::{dep_type, dep_obj};
 use macro_attr_2018::macro_attr;
-use dep_obj::flow::{Flows, FlowsMutex, Just};
+use dep_obj::binding::{Bindings, Binding1, Binding2};
 use dyn_context::{State, StateExt};
 use std::any::{TypeId, Any};
 use std::borrow::Cow;
 use std::fmt::Write;
 
-static FLOWS: FlowsMutex = FlowsMutex::new();
-
 macro_attr! {
     #[derive(Debug, Component!)]
-    struct ItemComponent {
+    struct ItemData {
         props: ItemProps,
     }
 }
 
-static ITEM_COMPONENT: ComponentClassMutex<ItemComponent> = ComponentClassMutex::new();
-
 macro_attr! {
     #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, NewtypeComponentId!)]
-    struct Item(Id<ItemComponent>);
+    struct Item(Id<ItemData>);
 }
 
 dep_type! {
@@ -41,7 +37,7 @@ dep_type! {
 impl Item {
     fn new(state: &mut dyn State) -> Item {
         let game: &mut Game = state.get_mut();
-        game.items.insert(|id| (ItemComponent { props: ItemProps::new_priv() }, Item(id)))
+        game.items.insert(|id| (ItemData { props: ItemProps::new_priv() }, Item(id)))
     }
 
     dep_obj! {
@@ -58,8 +54,6 @@ macro_attr! {
     }
 }
 
-static NPC_COMPONENT: ComponentClassMutex<NpcComponent> = ComponentClassMutex::new();
-
 macro_attr! {
     #[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash, NewtypeComponentId!)]
     struct Npc(Id<NpcComponent>);
@@ -76,13 +70,19 @@ impl Npc {
     fn new(state: &mut dyn State) -> Npc {
         let game: &mut Game = state.get_mut();
         let npc = game.npcs.insert(|id| (NpcComponent { props: NpcProps::new_priv() }, Npc(id)));
-        npc.props(state).vec(NpcProps::EQUIPPED_ITEMS)
-            .removed_inserted_items().handle(state, npc, |state, _npc, Just((removed, inserted))| {
-            for item in removed {
-                item.props(state).prop(ItemProps::EQUIPPED).set_distinct(false);
+        let removed_items_binding = Binding1::new(&mut game.bindings, |x| Some(x));
+        removed_items_binding.set_source_1(state, &mut NpcProps::EQUIPPED_ITEMS.removed_items_source(npc.props()));
+        removed_items_binding.handle_fn(state, (), |state, (), items| {
+            for item in items {
+                ItemProps::EQUIPPED.set_distinct(state, item.props(), false);
             }
-            for item in inserted {
-                item.props(state).prop(ItemProps::EQUIPPED).set_distinct(true);
+        });
+        let game: &mut Game = state.get_mut();
+        let inserted_items_binding = Binding1::new(&mut game.bindings, |x| Some(x));
+        inserted_items_binding.set_source_1(state, &mut NpcProps::EQUIPPED_ITEMS.inserted_items_source(npc.props()));
+        inserted_items_binding.handle_fn(state, (), |state, (), items| {
+            for item in items {
+                ItemProps::EQUIPPED.set_distinct(state, item.props(), true);
             }
         });
         npc
@@ -97,9 +97,9 @@ impl Npc {
 
 #[derive(Debug)]
 struct Game {
-    items: Arena<ItemComponent>,
+    items: Arena<ItemData>,
     npcs: Arena<NpcComponent>,
-    flows: Flows,
+    bindings: Bindings,
     log: String,
 }
 
@@ -107,8 +107,8 @@ impl State for Game {
     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
         if ty == TypeId::of::<Game>() {
             Some(self)
-        } else if ty == TypeId::of::<Flows>() {
-            Some(&self.flows)
+        } else if ty == TypeId::of::<Bindings>() {
+            Some(&self.bindings)
         } else {
             None
         }
@@ -117,8 +117,8 @@ impl State for Game {
     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
         if ty == TypeId::of::<Game>() {
             Some(self)
-        } else if ty == TypeId::of::<Flows>() {
-            Some(&mut self.flows)
+        } else if ty == TypeId::of::<Bindings>() {
+            Some(&mut self.bindings)
         } else {
             None
         }
@@ -128,9 +128,9 @@ impl State for Game {
 impl Game {
     fn new() -> Game {
         Game {
-            items: Arena::new(&mut ITEM_COMPONENT.lock().unwrap()),
-            npcs: Arena::new(&mut NPC_COMPONENT.lock().unwrap()),
-            flows: Flows::new(&mut FLOWS.lock().unwrap()),
+            items: Arena::new(),
+            npcs: Arena::new(),
+            bindings: Bindings::new(),
             log: String::new(),
         }
     }
@@ -140,21 +140,21 @@ fn main() {
     let game = &mut Game::new();
     let npc = Npc::new(game);
     let sword = Item::new(game);
-    sword.props(game).prop(ItemProps::NAME).set_uncond(Cow::Borrowed("Sword"));
+    ItemProps::NAME.set_uncond(game, sword.props(), Cow::Borrowed("Sword"));
     let shield = Item::new(game);
-    shield.props(game).prop(ItemProps::NAME).set_uncond(Cow::Borrowed("Shield"));
+    ItemProps::NAME.set_uncond(game, shield.props(), Cow::Borrowed("Shield"));
     for item in [sword, shield] {
-        item.props(game).prop(ItemProps::EQUIPPED).changes()
-            .zip(item.props(game).prop(ItemProps::NAME).values(), game)
-            .handle(game, item, |state, _item, Just(((old_equipped, equipped), name))| {
-                if old_equipped == equipped { return; }
-                let game: &mut Game = state.get_mut();
-                writeln!(&mut game.log, "{} {}.", name, if equipped { "equipped" } else { "unequipped" }).unwrap();
+        let log = Binding2::new(&mut game.bindings, |(old, new), (_, name)| if old == new { None} else { Some((new, name)) });
+        log.set_source_1(game, &mut ItemProps::EQUIPPED.source(item.props()));
+        log.set_source_2(game, &mut ItemProps::NAME.source(item.props()));
+        log.handle_fn(game, (), |game, (), (equipped, name)| {
+            let game: &mut Game = game.get_mut();
+            writeln!(&mut game.log, "{} {}.", name, if equipped { "equipped" } else { "unequipped" }).unwrap();
         });
     }
-    npc.props(game).vec(NpcProps::EQUIPPED_ITEMS).push(sword);
-    npc.props(game).vec(NpcProps::EQUIPPED_ITEMS).push(shield);
-    npc.props(game).vec(NpcProps::EQUIPPED_ITEMS).remove(0);
+    NpcProps::EQUIPPED_ITEMS.push(game, npc.props(), sword);
+    NpcProps::EQUIPPED_ITEMS.push(game, npc.props(), shield);
+    NpcProps::EQUIPPED_ITEMS.remove(game, npc.props(), 0);
     assert_eq!(game.log, "\
         Sword equipped.\n\
         Shield equipped.\n\
