@@ -6,14 +6,14 @@
 #![feature(const_raw_ptr_deref)]
 
 mod circuit {
-    use components_arena::{Component, NewtypeComponentId, Id, Arena, ComponentClassToken};
+    use components_arena::{Component, NewtypeComponentId, Id, Arena};
     use dep_obj::dep_obj;
     use downcast_rs::{Downcast, impl_downcast};
     use dyn_context::{SelfState};
     use macro_attr_2018::macro_attr;
     use std::fmt::Debug;
 
-    pub trait ChipLegs: Downcast + Debug + Send + Sync { }
+    pub trait ChipLegs: Downcast + Debug { }
 
     impl_downcast!(ChipLegs);
 
@@ -60,27 +60,14 @@ mod circuit {
     impl SelfState for Circuit { }
 
     impl Circuit {
-        pub fn new(token: &mut CircuitToken) -> Self {
-            Circuit {
-                arena: Arena::new(&mut token.0)
-            }
-        }
-    }
-
-    pub struct CircuitToken(ComponentClassToken<ChipNode>);
-
-    impl CircuitToken {
-        pub fn new() -> Option<Self> {
-            ComponentClassToken::new().map(CircuitToken)
-        }
+        pub fn new() -> Self { Circuit { arena: Arena::new() } }
     }
 }
 
 mod or_chip {
     use crate::circuit::*;
-    use components_arena::{ComponentId, RawId};
     use dep_obj::{dep_type};
-    use dep_obj::flow::{Just};
+    use dep_obj::binding::{Bindings, Binding2};
     use dyn_context::{State, StateExt};
 
     dep_type! {
@@ -97,16 +84,13 @@ mod or_chip {
             let legs = Self::new_priv();
             let circuit: &mut Circuit = state.get_mut();
             let chip = Chip::new(circuit, |chip| (Box::new(legs) as _, chip));
-            let in_1 = chip.legs(state).prop(OrLegs::IN_1).values();
-            let in_2 = chip.legs(state).prop(OrLegs::IN_2).values();
-            in_1.zip(in_2, state).handle(state, chip, Self::update);
+            let bindings: &mut Bindings = state.get_mut();
+            let binding = Binding2::new(bindings, |(_, in_1), (_, in_2)| Some(in_1 | in_2));
+            binding.set_source_1(state, &mut OrLegs::IN_1.source(chip.legs()));
+            binding.set_source_2(state, &mut OrLegs::IN_2.source(chip.legs()));
+            OrLegs::OUT.bind_distinct(state, chip.legs(), binding);
             chip
         }
-
-        fn update(state: &mut dyn State, chip: RawId, Just((in_1, in_2)): Just<(bool, bool)>) {
-            let chip = Chip::from_raw(chip);
-            chip.legs(state).prop(OrLegs::OUT).set_distinct(in_1 | in_2);
-        }
     }
 
     impl ChipLegs for OrLegs { }
@@ -114,9 +98,8 @@ mod or_chip {
 
 mod not_chip {
     use crate::circuit::*;
-    use components_arena::{ComponentId, RawId};
     use dep_obj::dep_type;
-    use dep_obj::flow::Just;
+    use dep_obj::binding::{Bindings, Binding1};
     use dyn_context::{State, StateExt};
 
     dep_type! {
@@ -132,22 +115,19 @@ mod not_chip {
             let circuit: &mut Circuit = state.get_mut();
             let legs = Self::new_priv();
             let chip = Chip::new(circuit, |chip| (Box::new(legs) as _, chip));
-            chip.legs(state).prop(NotLegs::IN_).values().handle(state, chip, Self::update);
+            let bindings: &mut Bindings = state.get_mut();
+            let binding = Binding1::new(bindings, |(_, in_1): (bool, bool)| Some(!in_1));
+            binding.set_source_1(state, &mut NotLegs::IN_.source(chip.legs()));
+            NotLegs::OUT.bind_distinct(state, chip.legs(), binding);
             chip
         }
-
-        fn update(state: &mut dyn State, chip: RawId, Just(in_): Just<bool>) {
-            let chip = Chip::from_raw(chip);
-            chip.legs(state).prop(NotLegs::OUT).set_distinct(!in_);
-        }
     }
 
     impl ChipLegs for NotLegs { }
 }
 
 use circuit::*;
-use components_arena::{ComponentId, RawId};
-use dep_obj::flow::{Flows, FlowsToken, Just};
+use dep_obj::binding::{Binding1, Bindings};
 use dyn_context::{State, StateRefMut};
 use not_chip::*;
 use or_chip::*;
@@ -163,15 +143,15 @@ struct TriggerChips {
 
 #[derive(Debug)]
 struct TriggerState {
-    flows: Flows,
+    bindings: Bindings,
     circuit: Circuit,
     chips: TriggerChips,
 }
 
 impl State for TriggerState {
     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
-        if ty == TypeId::of::<Flows>() {
-            Some(&self.flows)
+        if ty == TypeId::of::<Bindings>() {
+            Some(&self.bindings)
         } else if ty == TypeId::of::<Circuit>() {
             Some(&self.circuit)
         } else if ty == TypeId::of::<TriggerChips>() {
@@ -182,8 +162,8 @@ impl State for TriggerState {
     }
 
     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
-        if ty == TypeId::of::<Flows>() {
-            Some(&mut self.flows)
+        if ty == TypeId::of::<Bindings>() {
+            Some(&mut self.bindings)
         } else if ty == TypeId::of::<Circuit>() {
             Some(&mut self.circuit)
         } else {
@@ -193,45 +173,46 @@ impl State for TriggerState {
 }
 
 fn main() {
-    let mut circuit_token = CircuitToken::new().unwrap();
-    let mut circuit = Circuit::new(&mut circuit_token);
-    let mut flows_token = FlowsToken::new().unwrap();
-    let mut flows = Flows::new(&mut flows_token);
+    let mut circuit = Circuit::new();
+    let mut bindings = Bindings::new();
     let chips = (&mut circuit).merge_mut_and_then(|state| {
         let not_1 = NotLegs::new(state);
         let not_2 = NotLegs::new(state);
         let or_1 = OrLegs::new(state);
         let or_2 = OrLegs::new(state);
         TriggerChips { or_1, or_2, not_1, not_2 }
-    }, &mut flows);
+    }, &mut bindings);
     let state = &mut TriggerState {
         circuit,
-        flows,
+        bindings,
         chips: chips.clone(),
     };
-    let not_out_to_or_in = |state: &mut dyn State, or: RawId, Just(out): Just<bool>| {
-        let or = Chip::from_raw(or);
-        or.legs(state).prop(OrLegs::IN_2).set_uncond(out);
-    };
-    let or_out_to_not_out = |state: &mut dyn State, not: RawId, Just(out): Just<bool>| {
-        let not = Chip::from_raw(not);
-        not.legs(state).prop(NotLegs::IN_).set_uncond(out);
-    };
-    chips.not_1.legs(state).prop(NotLegs::OUT).values().handle(state, chips.or_2, not_out_to_or_in);
-    chips.not_2.legs(state).prop(NotLegs::OUT).values().handle(state, chips.or_1, not_out_to_or_in);
-    chips.or_1.legs(state).prop(OrLegs::OUT).values().handle(state, chips.not_1, or_out_to_not_out);
-    chips.or_2.legs(state).prop(OrLegs::OUT).values().handle(state, chips.not_2, or_out_to_not_out);
-    chips.not_1.legs(state).prop(NotLegs::OUT).changes().handle(state, (), |_, _, Just((old, new))| {
+    let not_1_out_to_or_2_in = Binding1::new(&mut state.bindings, |(_, value)| Some(value));
+    not_1_out_to_or_2_in.set_source_1(state, &mut NotLegs::OUT.source(chips.not_1.legs()));
+    OrLegs::IN_2.bind_uncond(state, chips.or_2.legs(), not_1_out_to_or_2_in);
+    let not_2_out_to_or_1_in = Binding1::new(&mut state.bindings, |(_, value)| Some(value));
+    not_2_out_to_or_1_in.set_source_1(state, &mut NotLegs::OUT.source(chips.not_2.legs()));
+    OrLegs::IN_2.bind_uncond(state, chips.or_1.legs(), not_2_out_to_or_1_in);
+    let or_1_out_to_not_1_in = Binding1::new(&mut state.bindings, |(_, value)| Some(value));
+    or_1_out_to_not_1_in.set_source_1(state, &mut OrLegs::OUT.source(chips.or_1.legs()));
+    NotLegs::IN_.bind_uncond(state, chips.not_1.legs(), or_1_out_to_not_1_in);
+    let or_2_out_to_not_2_in = Binding1::new(&mut state.bindings, |(_, value)| Some(value));
+    or_2_out_to_not_2_in.set_source_1(state, &mut OrLegs::OUT.source(chips.or_2.legs()));
+    NotLegs::IN_.bind_uncond(state, chips.not_2.legs(), or_2_out_to_not_2_in);
+
+    let print_out = Binding1::new(&mut state.bindings, |x| Some(x));
+    print_out.set_source_1(state, &mut NotLegs::OUT.source(chips.not_2.legs()));
+    print_out.handle_fn(state, (), |_, _, (old, new)| {
         let old = if old { "1" } else { "0" };
         let new = if new { "1" } else { "0" };
         println!("{} -> {}", old, new);
     });
-    chips.or_1.legs(state).prop(OrLegs::IN_1).set_distinct(true);
-    chips.or_1.legs(state).prop(OrLegs::IN_1).set_distinct(false);
-    chips.or_2.legs(state).prop(OrLegs::IN_1).set_distinct(true);
-    chips.or_2.legs(state).prop(OrLegs::IN_1).set_distinct(false);
-    chips.or_1.legs(state).prop(OrLegs::IN_1).set_distinct(true);
-    chips.or_1.legs(state).prop(OrLegs::IN_1).set_distinct(false);
-    chips.or_2.legs(state).prop(OrLegs::IN_1).set_distinct(true);
-    chips.or_2.legs(state).prop(OrLegs::IN_1).set_distinct(false);
+    OrLegs::IN_1.set_distinct(state, chips.or_1.legs(), true);
+    OrLegs::IN_1.set_distinct(state, chips.or_1.legs(), false);
+    OrLegs::IN_1.set_distinct(state, chips.or_2.legs(), true);
+    OrLegs::IN_1.set_distinct(state, chips.or_2.legs(), false);
+    OrLegs::IN_1.set_distinct(state, chips.or_1.legs(), true);
+    OrLegs::IN_1.set_distinct(state, chips.or_1.legs(), false);
+    OrLegs::IN_1.set_distinct(state, chips.or_2.legs(), true);
+    OrLegs::IN_1.set_distinct(state, chips.or_2.legs(), false);
 }
