@@ -13,15 +13,14 @@
 #![doc(test(attr(allow(dead_code))))]
 #![doc(test(attr(allow(unused_variables))))]
 
-#![cfg_attr(not(feature="std"), no_std)]
-#[cfg(feature="std")]
-extern crate core;
+#![no_std]
+
 extern crate alloc;
 
 mod base;
 pub use base::*;
 
-pub mod flow;
+pub mod binding;
 
 #[cfg(docsrs)]
 pub mod example {
@@ -130,6 +129,8 @@ pub use core::stringify as std_stringify;
 #[doc(hidden)]
 pub use dyn_context::State as dyn_context_State;
 #[doc(hidden)]
+pub use dyn_context::StateExt as dyn_context_StateExt;
+#[doc(hidden)]
 pub use generics::concat as generics_concat;
 #[doc(hidden)]
 pub use generics::parse as generics_parse;
@@ -138,27 +139,40 @@ pub use memoffset::offset_of as memoffset_offset_of;
 #[doc(hidden)]
 pub use paste::paste as paste_paste;
 
-use crate::flow::{Flow, FlowSource, Just, Through, Snd, RemovedInserted};
+use crate::binding::{Binding, Handler, Source, HandledSource, HandlerId};
 use alloc::boxed::Box;
 use alloc::collections::TryReserveError;
 use alloc::vec;
 use alloc::vec::Vec;
-use components_arena::{ComponentId, RawId};
+use components_arena::{Component, ComponentId, Arena, Id};
 use core::fmt::Debug;
 use core::mem::replace;
 use dyn_clone::{DynClone, clone_trait_object};
-use dyn_context::{State, StateExt};
+use dyn_context::State;
 use educe::Educe;
+use macro_attr_2018::macro_attr;
 use phantom_type::PhantomType;
 
 #[derive(Educe)]
-#[educe(Debug)]
+#[educe(Debug, Clone, Copy)]
+pub struct Glob<Id: ComponentId, Obj> {
+    pub id: Id,
+    #[educe(Debug(ignore))]
+    pub get_obj_mut: for<'a> fn(state: &'a mut dyn State, id: Id) -> &'a mut Obj,
+}
+
+macro_attr! {
+    #[derive(Debug, Clone, Component!(class=HandlerComponent))]
+    struct BoxedHandler<T: Convenient>(Box<dyn Handler<T>>);
+}
+
+#[derive(Debug)]
 pub struct DepPropEntry<PropType: Convenient> {
     default: &'static PropType,
     style: Option<PropType>,
     local: Option<PropType>,
-    #[educe(Debug(ignore))]
-    on_changed: Vec<(RawId, fn(state: &mut dyn State, handler_id: RawId, old_new: (PropType, PropType)))>,
+    handlers: Arena<BoxedHandler<(PropType, PropType)>>,
+    binding: Option<Binding<PropType>>,
 }
 
 impl<PropType: Convenient> DepPropEntry<PropType> {
@@ -167,25 +181,25 @@ impl<PropType: Convenient> DepPropEntry<PropType> {
             default,
             style: None,
             local: None,
-            on_changed: Vec::new()
+            handlers: Arena::new(),
+            binding: None,
         }
     }
 }
 
-
-#[derive(Educe)]
-#[educe(Debug)]
+#[derive(Debug)]
 pub struct DepVecEntry<ItemType: Convenient> {
     items: Vec<ItemType>,
-    #[educe(Debug(ignore))]
-    on_changed: Vec<(RawId, fn(state: &mut dyn State, handler_id: RawId, change_vec: (VecChange<ItemType>, Vec<ItemType>)))>,
+    removed_items_handlers: Arena<BoxedHandler<Vec<ItemType>>>,
+    inserted_items_handlers: Arena<BoxedHandler<Vec<ItemType>>>,
 }
 
 impl<ItemType: Convenient> DepVecEntry<ItemType> {
     pub const fn new() -> Self {
         DepVecEntry {
             items: Vec::new(),
-            on_changed: Vec::new()
+            removed_items_handlers: Arena::new(),
+            inserted_items_handlers: Arena::new(),
         }
     }
 }
@@ -200,9 +214,9 @@ impl<ItemType: Convenient> DepVecEntry<ItemType> {
 /// # #![feature(const_maybe_uninit_as_ptr)]
 /// # #![feature(const_ptr_offset_from)]
 /// # #![feature(const_raw_ptr_deref)]
-/// use components_arena::{Arena, Component, ComponentClassToken, NewtypeComponentId, Id};
+/// use components_arena::{Arena, Component, NewtypeComponentId, Id};
 /// use dep_obj::{dep_obj, dep_type};
-/// use dep_obj::flow::{Flows, FlowsToken, Just};
+/// use dep_obj::binding::{Bindings, Binding1};
 /// use dyn_context::{State, StateExt};
 /// use macro_attr_2018::macro_attr;
 /// use std::any::{Any, TypeId};
@@ -228,15 +242,15 @@ impl<ItemType: Convenient> DepVecEntry<ItemType> {
 /// }
 ///
 /// pub struct MyApp {
-///     flows: Flows,
+///     bindings: Bindings,
 ///     my_dep_types: Arena<MyDepTypePrivateData>,
 ///     res: i32,
 /// }
 ///
 /// impl State for MyApp {
 ///     fn get_raw(&self, ty: TypeId) -> Option<&dyn Any> {
-///         if ty == TypeId::of::<Flows>() {
-///             Some(&self.flows)
+///         if ty == TypeId::of::<Bindings>() {
+///             Some(&self.bindings)
 ///         } else if ty == TypeId::of::<MyApp>() {
 ///             Some(self)
 ///         } else {
@@ -245,8 +259,8 @@ impl<ItemType: Convenient> DepVecEntry<ItemType> {
 ///     }
 ///
 ///     fn get_mut_raw(&mut self, ty: TypeId) -> Option<&mut dyn Any> {
-///         if ty == TypeId::of::<Flows>() {
-///             Some(&mut self.flows)
+///         if ty == TypeId::of::<Bindings>() {
+///             Some(&mut self.bindings)
 ///         } else if ty == TypeId::of::<MyApp>() {
 ///             Some(self)
 ///         } else {
@@ -270,20 +284,20 @@ impl<ItemType: Convenient> DepVecEntry<ItemType> {
 /// }
 ///
 /// fn main() {
-///     let mut my_dep_types_token = ComponentClassToken::new().unwrap();
-///     let mut flows_token = FlowsToken::new().unwrap();
-///     let mut app = MyApp {
-///         flows: Flows::new(&mut flows_token),
-///         my_dep_types: Arena::new(&mut my_dep_types_token),
+///     let app = &mut MyApp {
+///         bindings: Bindings::new(),
+///         my_dep_types: Arena::new(),
 ///         res: 0,
 ///     };
-///     let id = MyDepTypeId::new(&mut app);
-///     id.obj(&mut app).prop(MyDepType::PROP_2).values().handle(&mut app, (), |state, _, Just(value)| {
-///         let app: &mut MyApp = state.get_mut();
+///     let id = MyDepTypeId::new(app);
+///     let res = Binding1::new(&mut app.bindings, |(_, x)| Some(x));
+///     res.set_source_1(app, &mut MyDepType::PROP_2.source(id.obj()));
+///     res.handle_fn(app, (), |app, (), value| {
+///         let app: &mut MyApp = app.get_mut();
 ///         app.res = value;
 ///     });
 ///     assert_eq!(app.res, 10);
-///     id.obj(&mut app).prop(MyDepType::PROP_2).set_distinct(5);
+///     MyDepType::PROP_2.set_distinct(app, id.obj(), 5);
 ///     assert_eq!(app.res, 5);
 /// }
 /// ```
@@ -311,14 +325,6 @@ impl<Owner: DepType, PropType: Convenient> DepProp<Owner, PropType> {
 
     pub fn offset(self) -> usize { self.offset }
 
-    fn entry(self, owner: &Owner) -> &DepPropEntry<PropType> {
-        unsafe {
-            let entry = (owner as *const _ as usize).unchecked_add(self.offset);
-            let entry = entry as *const DepPropEntry<PropType>;
-            &*entry
-        }
-    }
-
     fn entry_mut(self, owner: &mut Owner) -> &mut DepPropEntry<PropType> {
         unsafe {
             let entry = (owner as *mut _ as usize).unchecked_add(self.offset);
@@ -326,6 +332,121 @@ impl<Owner: DepType, PropType: Convenient> DepProp<Owner, PropType> {
             &mut *entry
         }
     }
+
+    pub fn set_uncond(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, value: PropType) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        let old = entry_mut.local.replace(value.clone()).unwrap_or_else(||
+            entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
+        );
+        for handler in entry_mut.handlers.items().clone().into_values() {
+            handler.0.execute(state, (old.clone(), value.clone()));
+        }
+    }
+
+    pub fn unset_uncond(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        if let Some(old) = entry_mut.local.take() {
+            let default = entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone();
+            for handler in entry_mut.handlers.items().clone().into_values() {
+                handler.0.execute(state, (old.clone(), default.clone()));
+            }
+        }
+    }
+
+    pub fn set_distinct(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, value: PropType) where PropType: PartialEq {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        let old = entry_mut.local.as_ref()
+            .or_else(|| entry_mut.style.as_ref())
+            .unwrap_or_else(|| entry_mut.default)
+            ;
+        if &value == old { return; }
+        let old = entry_mut.local.replace(value.clone()).unwrap_or_else(||
+            entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
+        );
+        for handler in entry_mut.handlers.items().clone().into_values() {
+            handler.0.execute(state, (old.clone(), value.clone()));
+        }
+    }
+
+    pub fn unset_distinct(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>) where PropType: PartialEq {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        if let Some(old) = entry_mut.local.take() {
+            let new = entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone();
+            if new == old { return; }
+            for handler in entry_mut.handlers.items().clone().into_values() {
+                handler.0.execute(state, (old.clone(), new.clone()));
+            }
+        }
+    }
+
+    fn bind_raw(
+        self,
+        state: &mut dyn State,
+        obj: Glob<Owner::Id, Owner>,
+        handler: impl FnOnce(Self, Glob<Owner::Id, Owner>) -> Box<dyn Handler<PropType>>,
+        binding: Binding<PropType>
+    ) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        if let Some(binding) = entry_mut.binding.replace(binding.clone()) {
+            binding.drop_binding(state);
+        }
+        binding.handle(state, handler(self, obj));
+    }
+
+    pub fn bind_distinct(
+        self,
+        state: &mut dyn State,
+        obj: Glob<Owner::Id, Owner>,
+        binding: impl Into<Binding<PropType>>
+    ) where PropType: PartialEq, Owner: 'static {
+        self.bind_raw(state, obj, |prop, obj| Box::new(DepPropSetDistinct { prop, obj }), binding.into());
+    }
+
+    pub fn bind_uncond(
+        self,
+        state: &mut dyn State,
+        obj: Glob<Owner::Id, Owner>,
+        binding: impl Into<Binding<PropType>>
+    ) where Owner: 'static {
+        self.bind_raw(state, obj, |prop, obj| Box::new(DepPropSetUncond { prop, obj }), binding.into());
+    }
+
+    pub fn unbind(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        if let Some(binding) = entry_mut.binding.take() {
+            binding.drop_binding(state);
+        }
+    }
+
+    pub fn source(self, obj: Glob<Owner::Id, Owner>) -> DepPropSource<Owner, PropType> {
+        DepPropSource { obj, prop: self }
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug, Clone)]
+struct DepPropSetDistinct<Owner: DepType, PropType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    prop: DepProp<Owner, PropType>,
+}
+
+impl<Owner: DepType, PropType: Convenient + PartialEq> Handler<PropType> for DepPropSetDistinct<Owner, PropType> {
+    fn execute(&self, state: &mut dyn State, value: PropType) {
+        self.prop.set_distinct(state, self.obj, value);
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug, Clone)]
+struct DepPropSetUncond<Owner: DepType, PropType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    prop: DepProp<Owner, PropType>,
+}
+
+impl<Owner: DepType, PropType: Convenient> Handler<PropType> for DepPropSetUncond<Owner, PropType> {
+    fn execute(&self, state: &mut dyn State, value: PropType) {
+        self.prop.set_uncond(state, self.obj, value);
+    }
 }
 
 #[derive(Educe)]
@@ -342,14 +463,6 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
 
     pub fn offset(self) -> usize { self.offset }
 
-    fn entry(self, owner: &Owner) -> &DepVecEntry<ItemType> {
-        unsafe {
-            let entry = (owner as *const _ as usize).unchecked_add(self.offset);
-            let entry = entry as *const DepVecEntry<ItemType>;
-            &*entry
-        }
-    }
-
     fn entry_mut(self, owner: &mut Owner) -> &mut DepVecEntry<ItemType> {
         unsafe {
             let entry = (owner as *mut _ as usize).unchecked_add(self.offset);
@@ -357,6 +470,54 @@ impl<Owner: DepType, ItemType: Convenient> DepVec<Owner, ItemType> {
             &mut *entry
         }
     }
+
+    pub fn clear(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        let items = replace(&mut entry_mut.items, Vec::new());
+        for handler in entry_mut.removed_items_handlers.items().clone().into_values() {
+            handler.0.execute(state, items.clone());
+        }
+    }
+
+    pub fn push(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, item: ItemType) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        entry_mut.items.push(item.clone());
+        for handler in entry_mut.inserted_items_handlers.items().clone().into_values() {
+            handler.0.execute(state, vec![item.clone()]);
+        }
+    }
+
+    pub fn insert(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, index: usize, item: ItemType) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        entry_mut.items.insert(index, item.clone());
+        for handler in entry_mut.inserted_items_handlers.items().clone().into_values() {
+            handler.0.execute(state, vec![item.clone()]);
+        }
+    }
+
+    pub fn remove(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, index: usize) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        let item = entry_mut.items.remove(index);
+        for handler in entry_mut.removed_items_handlers.items().clone().into_values() {
+            handler.0.execute(state, vec![item.clone()]);
+        }
+    }
+
+    pub fn extend_from_slice(self, state: &mut dyn State, obj: Glob<Owner::Id, Owner>, other: &[ItemType]) {
+        let entry_mut = self.entry_mut((obj.get_obj_mut)(state, obj.id));
+        entry_mut.items.extend_from_slice(other);
+        for handler in entry_mut.inserted_items_handlers.items().clone().into_values() {
+            handler.0.execute(state, Vec::from(other));
+        }
+    }
+
+    pub fn inserted_items_source(self, obj: Glob<Owner::Id, Owner>) -> DepVecInsertedItemSource<Owner, ItemType> {
+        DepVecInsertedItemSource { obj, vec: self }
+    }
+
+    pub fn removed_items_source(self, obj: Glob<Owner::Id, Owner>) -> DepVecRemovedItemSource<Owner, ItemType> {
+        DepVecRemovedItemSource { obj, vec: self }
+    }
 }
 
 #[derive(Educe)]
@@ -366,7 +527,7 @@ struct Setter<Owner: DepType, PropType: Convenient> {
     value: PropType,
 }
 
-trait AnySetter<Owner: DepType>: Debug + DynClone + Send + Sync {
+trait AnySetter<Owner: DepType>: Debug + DynClone {
     fn prop_offset(&self) -> usize;
     fn un_apply(
         &self,
@@ -386,28 +547,30 @@ impl<Owner: DepType, PropType: Convenient> AnySetter<Owner> for Setter<Owner, Pr
         unapply: bool
     ) -> Option<Box<dyn for<'a> FnOnce(&'a mut dyn State)>> {
         let entry_mut = self.prop.entry_mut(owner);
-        let on_changed = if entry_mut.local.is_some() {
+        let handlers = if entry_mut.local.is_some() {
             None
         } else {
-            Some(entry_mut.on_changed.clone())
+            Some(entry_mut.handlers.items().clone())
         };
         let value = if unapply { None } else { Some(self.value.clone()) };
         let old = replace(&mut entry_mut.style, value.clone());
-        on_changed.map(|on_changed| {
+        if let Some(handlers) = handlers {
             let old = old.unwrap_or_else(|| entry_mut.default.clone());
             let value = value.unwrap_or_else(|| entry_mut.default.clone());
-            Box::new(move |state: &'_ mut dyn State| {
-                for (handler_id, handler) in on_changed {
-                    handler(state, handler_id, (old.clone(), value.clone()));
+            Some(Box::new(move |state: &'_ mut dyn State| {
+                for handler in handlers.into_values() {
+                    handler.0.execute(state, (old.clone(), value.clone()));
                 }
-            }) as _
-        })
+            }) as _)
+        } else {
+            None
+        }
     }
 }
 
 /// A dictionary mapping a subset of target type properties to the values.
 /// Every dependency object can have an applied style at every moment.
-/// To switch an applied style, use the [`DepObj::apply_style`] function.
+/// To switch an applied style, use the [`OptionStyleExt::apply`] function.
 #[derive(Educe)]
 #[educe(Debug, Clone, Default)]
 pub struct Style<Owner: DepType> {
@@ -461,242 +624,27 @@ impl<Owner: DepType> Style<Owner> {
     }
 }
 
-pub trait DepObjBuilderCore<OwnerId: ComponentId> {
-    fn state(&self) -> &dyn State;
-    fn state_mut(&mut self) -> &mut dyn State;
-    fn id(&self) -> OwnerId;
+pub trait OptionStyleExt<Owner: DepType> {
+    fn apply(
+        self,
+        state: &mut dyn State,
+        obj: Glob<Owner::Id, Owner>,
+    ) -> Option<Style<Owner>>;
 }
 
-pub struct DepObjProp<'a, 'b, Owner: DepType, Arena, PropType: Convenient> {
-    obj: &'b mut DepObj<'a, Owner, Arena>,
-    prop: DepProp<Owner, PropType>,
-}
-
-impl<'a, 'b, Owner: DepType, Arena: 'static, PropType: Convenient> DepObjProp<'a, 'b, Owner, Arena, PropType> {
-    pub fn values(&mut self) -> Flow<Just<PropType>> {
-        Flow::new_through(<Through<Snd<PropType>>>::new(), self)
-    }
-
-    pub fn changes(&mut self) -> Flow<Just<(PropType, PropType)>> {
-        Flow::new(self)
-    }
-
-    pub fn set_uncond(&mut self, value: PropType) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.prop.entry_mut(obj);
-        let old = entry_mut.local.replace(value.clone()).unwrap_or_else(||
-            entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
-        );
-        for (hanlder_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, hanlder_id, (old.clone(), value.clone()));
-        }
-    }
-
-    pub fn unset_uncond(&mut self) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.prop.entry_mut(obj);
-        if let Some(old) = entry_mut.local.take() {
-            let default = entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone();
-            for (handler_id, handler) in entry_mut.on_changed.clone() {
-                handler(self.obj.state, handler_id, (old.clone(), default.clone()));
-            }
-        }
-    }
-
-    pub fn set_distinct(&mut self, value: PropType) where PropType: PartialEq {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.prop.entry_mut(obj);
-        let old = entry_mut.local.as_ref()
-            .or_else(|| entry_mut.style.as_ref())
-            .unwrap_or_else(|| entry_mut.default)
-        ;
-        if &value == old { return; }
-        let old = entry_mut.local.replace(value.clone()).unwrap_or_else(||
-            entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone()
-        );
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, (old.clone(), value.clone()));
-        }
-    }
-
-    pub fn unset_distinct(&mut self) where PropType: PartialEq {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.prop.entry_mut(obj);
-        if let Some(old) = entry_mut.local.take() {
-            let new = entry_mut.style.as_ref().unwrap_or_else(|| entry_mut.default).clone();
-            if new == old { return; }
-            for (handler_id, handler) in entry_mut.on_changed.clone() {
-                handler(self.obj.state, handler_id, (old.clone(), new.clone()));
-            }
-        }
-    }
-}
-
-impl<'a, 'b, Owner: DepType, Arena: 'static, PropType: Convenient> FlowSource for DepObjProp<'a, 'b, Owner, Arena, PropType> {
-    type Value = (PropType, PropType);
-
-    fn handle<Id:ComponentId, R>(
-        &mut self,
-        handler: impl FnOnce(
-            Self::Value,
-            &mut dyn State
-        ) -> (Id, fn(state: &mut dyn State, handler_id: RawId, value: Self::Value), R),
-    ) -> R {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry = self.prop.entry(obj);
-        let old = entry.default.clone();
-        let new = entry.local.as_ref().or_else(|| entry.style.as_ref()).unwrap_or_else(|| entry.default).clone();
-        let (handler_id, handler, res) = handler((old, new), self.obj.state);
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.prop.entry_mut(obj);
-        entry_mut.on_changed.push((handler_id.into_raw(), handler));
-        res
-    }
-}
-
-pub struct DepObjVec<'a, 'b, Owner: DepType, Arena, ItemType: Convenient> {
-    obj: &'b mut DepObj<'a, Owner, Arena>,
-    vec: DepVec<Owner, ItemType>,
-}
-
-impl<'a, 'b, Owner: DepType, Arena: 'static, ItemType: Convenient> DepObjVec<'a, 'b, Owner, Arena, ItemType> {
-    pub fn changes(&mut self) -> Flow<Just<(VecChange<ItemType>, Vec<ItemType>)>> {
-        Flow::new(self)
-    }
-
-    pub fn removed_inserted_items(&mut self) -> Flow<Just<(Vec<ItemType>, Vec<ItemType>)>> {
-        Flow::new_through(<Through<RemovedInserted<ItemType>>>::new(), self)
-    }
-
-    pub fn clear(&mut self) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        let old_items = replace(&mut entry_mut.items, Vec::new());
-        let change_vec = (VecChange::Reset(old_items), Vec::new());
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, change_vec.clone());
-        }
-    }
-
-    pub fn push(&mut self, item: ItemType) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        entry_mut.items.push(item);
-        let change = VecChange::Inserted(
-            unsafe { entry_mut.items.len().unchecked_sub(1) } .. entry_mut.items.len()
-        );
-        let vec = entry_mut.items.clone();
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, (change.clone(), vec.clone()));
-        }
-    }
-
-    pub fn insert(&mut self, index: usize, item: ItemType) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        entry_mut.items.insert(index, item);
-        let change = VecChange::Inserted(index .. index + 1);
-        let vec = entry_mut.items.clone();
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, (change.clone(), vec.clone()));
-        }
-    }
-
-    pub fn remove(&mut self, index: usize) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        let item = entry_mut.items.remove(index);
-        let change = VecChange::Removed(index, vec![item]);
-        let vec = entry_mut.items.clone();
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, (change.clone(), vec.clone()));
-        }
-    }
-
-    pub fn append(&mut self, other: &mut Vec<ItemType>) {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        let appended = other.len();
-        entry_mut.items.append(other);
-        let change = VecChange::Inserted(
-            unsafe { entry_mut.items.len().unchecked_sub(appended) } .. entry_mut.items.len()
-        );
-        let vec = entry_mut.items.clone();
-        for (handler_id, handler) in entry_mut.on_changed.clone() {
-            handler(self.obj.state, handler_id, (change.clone(), vec.clone()));
-        }
-    }
-}
-
-impl<'a, 'b, Owner: DepType, Arena: 'static, ItemType: Convenient> FlowSource for DepObjVec<'a, 'b, Owner, Arena, ItemType> {
-    type Value = (VecChange<ItemType>, Vec<ItemType>);
-
-    fn handle<Id:ComponentId, R>(
-        &mut self,
-        handler: impl FnOnce(
-            Self::Value,
-            &mut dyn State
-        ) -> (Id, fn(state: &mut dyn State, handler_id: RawId, value: Self::Value), R),
-    ) -> R {
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry = self.vec.entry(obj);
-        let items = entry.items.clone();
-        let (handler_id, handler, res) = handler((VecChange::Reset(Vec::new()), items), self.obj.state);
-        let arena: &mut Arena = self.obj.state.get_mut();
-        let obj = (self.obj.get_obj_mut)(arena, self.obj.id);
-        let entry_mut = self.vec.entry_mut(obj);
-        entry_mut.on_changed.push((handler_id.into_raw(), handler));
-        res
-    }
-}
-
-pub struct DepObj<'a, Owner: DepType, Arena> {
-    id: Owner::Id,
-    state: &'a mut dyn State,
-    get_obj_mut: for<'b> fn(arena: &'b mut Arena, id: Owner::Id) -> &'b mut Owner,
-}
-
-impl<'a, Owner: DepType, Arena: 'static> DepObj<'a, Owner, Arena> {
-    pub fn new(
-        id: Owner::Id,
-        state: &'a mut dyn State,
-        get_obj_mut: for<'b> fn(arena: &'b mut Arena, id: Owner::Id) -> &'b mut Owner,
-    ) -> Self {
-        DepObj { id, state, get_obj_mut }
-    }
-
-    pub fn prop<'b, PropType: Convenient>(&'b mut self, prop: DepProp<Owner, PropType>) -> DepObjProp<'a, 'b, Owner, Arena, PropType> {
-        DepObjProp { obj: self, prop }
-    }
-
-    pub fn vec<'b, ItemType: Convenient>(&'b mut self, vec: DepVec<Owner, ItemType>) -> DepObjVec<'a, 'b, Owner, Arena, ItemType> {
-        DepObjVec { obj: self, vec }
-    }
-
-    pub fn apply_style(
-        &mut self,
-        style: Option<Style<Owner>>,
+impl<Owner: DepType> OptionStyleExt<Owner> for Option<Style<Owner>> {
+    fn apply(
+        self,
+        state: &mut dyn State,
+        obj: Glob<Owner::Id, Owner>,
     ) -> Option<Style<Owner>> {
-        let arena: &mut Arena = self.state.get_mut();
-        let obj = (self.get_obj_mut)(arena, self.id);
+        let obj = (obj.get_obj_mut)(state, obj.id);
         let mut on_changed = Vec::new();
         let old = obj.style__().take();
         if let Some(old) = old.as_ref() {
             old.setters
                 .iter()
-                .filter(|setter| style.as_ref().map_or(
+                .filter(|setter| self.as_ref().map_or(
                     true,
                     |new| new.setters.binary_search_by_key(
                         &setter.prop_offset(),
@@ -707,21 +655,133 @@ impl<'a, Owner: DepType, Arena: 'static> DepObj<'a, Owner, Arena> {
                 .for_each(|x| on_changed.push(x))
             ;
         }
-        if let Some(new) = style.as_ref() {
+        if let Some(new) = self.as_ref() {
             new.setters
                 .iter()
                 .filter_map(|setter| setter.un_apply(obj, false))
                 .for_each(|x| on_changed.push(x))
             ;
         }
-        *obj.style__() = style;
+        *obj.style__() = self;
         for on_changed in on_changed {
-            on_changed(self.state);
+            on_changed(state);
         }
         old
     }
 }
 
+pub trait DepObjBuilderCore<OwnerId: ComponentId> {
+    fn state(&self) -> &dyn State;
+    fn state_mut(&mut self) -> &mut dyn State;
+    fn id(&self) -> OwnerId;
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+struct DepPropHandledSource<Owner: DepType, PropType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    handler_id: Id<BoxedHandler<(PropType, PropType)>>,
+    prop: DepProp<Owner, PropType>,
+}
+
+impl<Owner: DepType, PropType: Convenient> HandlerId for DepPropHandledSource<Owner, PropType> {
+    fn unhandle(&self, state: &mut dyn State) {
+        let entry_mut = self.prop.entry_mut((self.obj.get_obj_mut)(state, self.obj.id));
+        entry_mut.handlers.remove(self.handler_id);
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+pub struct DepPropSource<Owner: DepType, PropType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    prop: DepProp<Owner, PropType>,
+}
+
+impl<Owner: DepType + 'static, PropType: Convenient> Source<(PropType, PropType)> for DepPropSource<Owner, PropType> {
+    fn handle(&self, state: &mut dyn dyn_context_State, handler: Box<dyn Handler<(PropType, PropType)>>) -> HandledSource<(PropType, PropType)> {
+        let obj = (self.obj.get_obj_mut)(state, self.obj.id);
+        let entry = self.prop.entry_mut(obj);
+        let old = entry.default.clone();
+        let new = entry.local.as_ref().or_else(|| entry.style.as_ref()).unwrap_or_else(|| entry.default).clone();
+        let handler_id = entry.handlers.insert(|handler_id| (BoxedHandler(handler), handler_id));
+        HandledSource {
+            handler_id: Box::new(DepPropHandledSource { handler_id, obj: self.obj, prop: self.prop }),
+            value: (old, new)
+        }
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+struct DepVecInsertedItemsHandledSource<Owner: DepType, ItemType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    handler_id: Id<BoxedHandler<Vec<ItemType>>>,
+    vec: DepVec<Owner, ItemType>,
+}
+
+impl<Owner: DepType, ItemType: Convenient> HandlerId for DepVecInsertedItemsHandledSource<Owner, ItemType> {
+    fn unhandle(&self, state: &mut dyn State) {
+        let entry_mut = self.vec.entry_mut((self.obj.get_obj_mut)(state, self.obj.id));
+        entry_mut.inserted_items_handlers.remove(self.handler_id);
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+struct DepVecRemovedItemsHandledSource<Owner: DepType, ItemType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    handler_id: Id<BoxedHandler<Vec<ItemType>>>,
+    vec: DepVec<Owner, ItemType>,
+}
+
+impl<Owner: DepType, ItemType: Convenient> HandlerId for DepVecRemovedItemsHandledSource<Owner, ItemType> {
+    fn unhandle(&self, state: &mut dyn State) {
+        let entry_mut = self.vec.entry_mut((self.obj.get_obj_mut)(state, self.obj.id));
+        entry_mut.removed_items_handlers.remove(self.handler_id);
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+pub struct DepVecInsertedItemSource<Owner: DepType, ItemType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    vec: DepVec<Owner, ItemType>,
+}
+
+impl<Owner: DepType + 'static, ItemType: Convenient> Source<Vec<ItemType>> for DepVecInsertedItemSource<Owner, ItemType> {
+    fn handle(&self, state: &mut dyn dyn_context_State, handler: Box<dyn Handler<Vec<ItemType>>>) -> HandledSource<Vec<ItemType>> {
+        let obj = (self.obj.get_obj_mut)(state, self.obj.id);
+        let entry = self.vec.entry_mut(obj);
+        let items = entry.items.clone();
+        let handler_id = entry.inserted_items_handlers.insert(|handler_id| (BoxedHandler(handler), handler_id));
+        HandledSource {
+            handler_id: Box::new(DepVecInsertedItemsHandledSource { handler_id, obj: self.obj, vec: self.vec }),
+            value: items
+        }
+    }
+}
+
+#[derive(Educe)]
+#[educe(Debug)]
+pub struct DepVecRemovedItemSource<Owner: DepType, ItemType: Convenient> {
+    obj: Glob<Owner::Id, Owner>,
+    vec: DepVec<Owner, ItemType>,
+}
+
+impl<Owner: DepType + 'static, ItemType: Convenient> Source<Vec<ItemType>> for DepVecRemovedItemSource<Owner, ItemType> {
+    fn handle(&self, state: &mut dyn dyn_context_State, handler: Box<dyn Handler<Vec<ItemType>>>) -> HandledSource<Vec<ItemType>> {
+        let obj = (self.obj.get_obj_mut)(state, self.obj.id);
+        let entry = self.vec.entry_mut(obj);
+        let items = entry.items.clone();
+        let handler_id = entry.removed_items_handlers.insert(|handler_id| (BoxedHandler(handler), handler_id));
+        HandledSource {
+            handler_id: Box::new(DepVecRemovedItemsHandledSource { handler_id, obj: self.obj, vec: self.vec }),
+            value: items
+        }
+    }
+}
+
 #[macro_export]
 macro_rules! dep_type_with_builder {
     (
@@ -1307,19 +1367,20 @@ macro_rules! dep_obj {
         }
     ) => {
         $crate::paste_paste! {
-            fn [< $name _get_obj_priv >] <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                $arena: &'arena_lifetime mut $Arena,
+            #[allow(dead_code)]
+            $vis fn [< $name _mut >] <'state_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
+                state: &'state_lifetime mut dyn $crate::dyn_context_State,
                 $this: Self
-            ) -> &'arena_lifetime mut DepObjType {
-                $field_mut.downcast_mut::<DepObjType>().expect("invalid cast")
+            ) -> &'state_lifetime mut DepObjType {
+                let $arena: &mut $Arena = <dyn $crate::dyn_context_State as $crate::dyn_context_StateExt>::get_mut(state);
+                ($field_mut).downcast_mut::<DepObjType>().expect("invalid cast")
             }
 
             #[allow(dead_code)]
-            $vis fn $name <'arena_lifetime, DepObjType: $ty + $crate::DepType<Id=Self>>(
-                self,
-                state: &'arena_lifetime mut dyn $crate::dyn_context_State,
-            ) -> $crate::DepObj<'arena_lifetime, DepObjType, $Arena> {
-                $crate::DepObj::new(self, state, Self:: [< $name _get_obj_priv >] )
+            $vis fn $name <DepObjType: $ty + $crate::DepType<Id=Self>>(
+                self
+            ) -> $crate::Glob<Self, DepObjType> {
+                $crate::Glob { id: self, get_obj_mut: Self:: [< $name _mut >] }
             }
         }
     };
@@ -1329,19 +1390,20 @@ macro_rules! dep_obj {
         }
     ) => {
         $crate::paste_paste! {
-            fn [< $name _get_obj_priv >] <'arena_lifetime>(
-                $arena: &'arena_lifetime mut $Arena,
+            #[allow(dead_code)]
+            pub fn [< $name _mut >] <'state_lifetime>(
+                state: &'state_lifetime mut dyn $crate::dyn_context_State,
                 $this: Self
-            ) -> &'arena_lifetime mut $ty {
+            ) -> &'state_lifetime mut $ty {
+                let $arena: &mut $Arena = <dyn $crate::dyn_context_State as $crate::dyn_context_StateExt>::get_mut(state);
                 $field_mut
             }
 
             #[allow(dead_code)]
-            $vis fn $name <'arena_lifetime>(
-                self,
-                state: &'arena_lifetime mut dyn $crate::dyn_context_State,
-            ) -> $crate::DepObj<'arena_lifetime, $ty, $Arena> {
-                $crate::DepObj::new(self, state, Self:: [< $name _get_obj_priv >] )
+            $vis fn $name (
+                self
+            ) -> $crate::Glob<Self, $ty> {
+                $crate::Glob { id: self, get_obj_mut: Self:: [< $name _mut >] }
             }
         }
     };
